==> pydantic_base_aiogram-1.0.130-py3-none-any.whl: pydantic_base_aiogram/view/base.py <==
from typing import Any, Callable, Iterable, Optional, Union
from pydantic.fields import ModelField
from aiogram import F, Router
from aiogram.fsm.context import FSMContext
from aiogram.utils.keyboard import InlineKeyboardBuilder
from aiogram.fsm.state import State

from pydantic_base_aiogram.types import DataGetterCallable, Event
from pydantic_base_aiogram.abstract_handler import AbstractPydanticFormHandlers as THandler
from pydantic_base_aiogram.field_factory import logger
from pydantic_base_aiogram.utils.abstractions import is_list_type
from .abstract import AbstractView


class BaseView(AbstractView):
    _IGNORE_LIST_BUTTON = False

    def __init__(
        self, 
        state: State,
        filters: Iterable = tuple(),
        extra_keys: Optional[dict[str, str]] = None,
        getter: Optional[DataGetterCallable] = None,
        item_callback_data: Optional[str] = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs, name_format="{step_name}_view")
        self.item_callback_data = item_callback_data or "ielem"
        self.state = state
        self.filters = filters
        self.callback_data = self._get_callback_data()
        self._extra_keys = extra_keys or self.field.field_info.extra.get('extra_keys')
        self.getter = getter or self.field.field_info.extra.get('getter')
        self.is_static_keyboard = self.getter is None
        self.keyboard_page_size = 10
        self.keyboard = (
            self._build_base_keyboard() 
            if (
                self.is_static_keyboard 
                and self.back_data_getter is None 
            )
            else None
        )
        self.field_name = self.field.field_info.extra.get('short_name') or self.field.name
        self.text = (
            self.field.field_info.extra.get('view_text') 
            or self.view_text_format
        ).format(
            field_name=self.field_name,
            separator=self._text_seperator_symbol[-1]
        )

        logger.debug(f"[{self.__class__.__name__}][__init__]: {locals()=};")

    @property
    def view_text_format(self):
        return self.dialects.INPUT_STR

    @property
    def extra_keys(self) -> dict[str, str]:
        return (self._extra_keys and self._extra_keys.copy()) or {}

    async def get_dynamic_keyboard(self, state: FSMContext, page: int = 1, builder: Optional[InlineKeyboardBuilder] = None):
        builder = builder or InlineKeyboardBuilder()

        if self.getter is not None:
            for data, text in (await self.getter(state, page, self.keyboard_page_size)).items():
                cd = f"{self.item_callback_data}:{data}"
                builder.button(
                    text=text, 
                    callback_data=cd,
                )

        return self._build_base_keyboard(builder)

    def _build_base_keyboard(self, builder: Optional[InlineKeyboardBuilder] = None):
        builder = builder or InlineKeyboardBuilder()

        for data, text in self.extra_keys.items():
            if str(data).lower().startswith(f'{self.item_callback_data}:http') and (ls := str(text).split('*')) and len(ls) > 1:
                builder.button(text=ls[0], url=ls[-1])
            else:
                builder.button(text=text, callback_data=data)

        if not self.field.required:
            builder.button(
                text=self.dialects.SKIP_BUTTON,
                callback_data=f"{self.base_cq_prefix}_{self.dialects.SKIP_STEP_DATA}",
            )

        if is_list_type(self.field.outer_type_) and not self._IGNORE_LIST_BUTTON:
            builder.button(
                text=self.dialects.READY_BUTTON, 
                callback_data=self.dialects.READY_BUTTON_DATA, 
            )

        if (self.is_has_back or self.back_data) and self.back_allowed:
            builder.button(
                text=self.dialects.BACK_BUTTON, 
                callback_data=(
                    self.back_data
                    or f"{self.base_cq_prefix}_{self.dialects.BACK_BUTTON_DATA}"
                )
            )

        return builder.adjust(1)

    def _get_callback_data(self) -> str:
        top_levels = '.'.join(self.parents or tuple())
        return f"{top_levels}.{self.field.name}"

    def register2router(self, router: Router) -> Router:
        router.callback_query(F.data.startswith(self.callback_data), *self.filters)(self.__call__)

        logger.debug(f"[{self.__class__.__name__}][register2router]: {locals()=};")
        return router

    async def __call__(self, self_: THandler, event, state: FSMContext) -> Any:
        event = Event(event)
        res = await self.main(self_, event, state)
        await self._set_current_step(state)
        return res

    async def _set_current_step(self, state: FSMContext):
        await state.update_data(__step__=self.step_name)

    async def main(self, self_: THandler, event: Event, state: FSMContext) -> Any:
        reply_markup=(  # type: ignore
            self.keyboard if self.keyboard
            else await self.get_dynamic_keyboard(state)
        ).as_markup()

        if not reply_markup or not reply_markup.inline_keyboard:  # type: ignore
            reply_markup = None

        await event.answer(
            text=self.text, 
            state=state, 
            reply_markup=reply_markup,  # type: ignore
        )
        await state.set_state(self.state)

    @classmethod
    def create(cls, field: ModelField, **kwargs) -> 'BaseView':
        logger.debug(f"[{cls.__name__}][create]: {field.name=};")
        return cls(field=field, **kwargs)

==> pydantic_base_aiogram-1.0.135-py3-none-any.whl: pydantic_base_aiogram/view/base.py <==
from typing import Any, Callable, Iterable, Optional, Union
from pydantic.fields import ModelField
from aiogram import F, Router
from aiogram.fsm.context import FSMContext
from aiogram.utils.keyboard import InlineKeyboardBuilder
from aiogram.fsm.state import State

from pydantic_base_aiogram.types import DataGetterCallable, Event
from pydantic_base_aiogram.abstract_handler import AbstractPydanticFormHandlers as THandler
from pydantic_base_aiogram.field_factory import logger
from pydantic_base_aiogram.utils.abstractions import is_list_type
from .abstract import AbstractView


class BaseView(AbstractView):
    _IGNORE_LIST_BUTTON = False

    def __init__(
        self, 
        state: State,
        filters: Iterable = tuple(),
        extra_keys: Optional[dict[str, str]] = None,
        getter: Optional[DataGetterCallable] = None,
        item_callback_data: Optional[str] = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs, name_format="{step_name}_view")
        self.item_callback_data = item_callback_data or "ielem"
        self.state = state
        self.filters = filters
        self.callback_data = self._get_callback_data()
        self._extra_keys = extra_keys or self.field.field_info.extra.get('extra_keys')
        self.getter = getter or self.field.field_info.extra.get('getter')
        self.is_static_keyboard = not self._is_state_base_manage and self.getter is None
        self.keyboard_page_size = 10
        self.keyboard = (
            self._build_base_keyboard() 
            if (
                self.is_static_keyboard 
                and self.back_data_getter is None 
            )
            else None
        )
        self.field_name = self.field.field_info.extra.get('short_name') or self.field.name
        self.text = (
            self.field.field_info.extra.get('view_text') 
            or self.view_text_format
        ).format(
            field_name=self.field_name,
            separator=self._text_seperator_symbol[-1]
        )

        logger.debug(f"[{self.__class__.__name__}][__init__]: {locals()=};")

    @property
    def view_text_format(self):
        return self.dialects.INPUT_STR

    @property
    def extra_keys(self) -> dict[str, str]:
        return (self._extra_keys and self._extra_keys.copy()) or {}

    async def get_dynamic_keyboard(self, state: FSMContext, page: int = 1, builder: Optional[InlineKeyboardBuilder] = None):
        builder = builder or InlineKeyboardBuilder()

        if self.getter is not None:
            for data, text in (await self.getter(state, page, self.keyboard_page_size)).items():
                cd = f"{self.item_callback_data}:{data}"
                builder.button(
                    text=text, 
                    callback_data=cd,
                )

        back_cq = None

        if self._is_state_base_manage:
            state_data = await state.get_data()
            back_cq = state_data.pop('cback_cq', None)

        return self._build_base_keyboard(builder, back_cq=back_cq)

    def _build_base_keyboard(self, builder: Optional[InlineKeyboardBuilder] = None, *, back_cq = None):
        builder = builder or InlineKeyboardBuilder()

        for data, text in self.extra_keys.items():
            if str(data).lower().startswith(f'{self.item_callback_data}:http') and (ls := str(text).split('*')) and len(ls) > 1:
                builder.button(text=ls[0], url=ls[-1])
            else:
                builder.button(text=text, callback_data=data)

        if not self.field.required:
            builder.button(
                text=self.dialects.SKIP_BUTTON,
                callback_data=f"{self.base_cq_prefix}_{self.dialects.SKIP_STEP_DATA}",
            )

        if is_list_type(self.field.outer_type_) and not self._IGNORE_LIST_BUTTON:
            builder.button(
                text=self.dialects.READY_BUTTON, 
                callback_data=self.dialects.READY_BUTTON_DATA, 
            )

        back_cq = back_cq or (
            self.back_allowed 
            and (
                (
                    self.is_has_back
                    and f"{self.base_cq_prefix}_{self.dialects.BACK_BUTTON_DATA}"
                )
                or self.back_data 
            )
        )

        if back_cq:
            builder.button(
                text=self.dialects.BACK_BUTTON, 
                callback_data=back_cq,
            )

        return builder.adjust(1)

    def _get_callback_data(self) -> str:
        top_levels = '.'.join(self.parents or tuple())
        return f"{top_levels}.{self.field.name}"

    def register2router(self, router: Router) -> Router:
        router.callback_query(F.data.startswith(self.callback_data), *self.filters)(self.__call__)

        logger.debug(f"[{self.__class__.__name__}][register2router]: {locals()=};")
        return router

    async def __call__(self, self_: THandler, event, state: FSMContext) -> Any:
        event = Event(event)
        res = await self.main(self_, event, state)
        await self._set_current_step(state)
        return res

    async def _set_current_step(self, state: FSMContext):
        await state.update_data(__step__=self.step_name)

    async def main(self, self_: THandler, event: Event, state: FSMContext) -> Any:
        reply_markup=(  # type: ignore
            self.keyboard if self.keyboard
            else await self.get_dynamic_keyboard(state)
        ).as_markup()

        if not reply_markup or not reply_markup.inline_keyboard:  # type: ignore
            reply_markup = None

        await event.answer(
            text=self.text, 
            state=state, 
            reply_markup=reply_markup,  # type: ignore
        )
        await state.set_state(self.state)

    @classmethod
    def create(cls, field: ModelField, **kwargs) -> 'BaseView':
        logger.debug(f"[{cls.__name__}][create]: {field.name=};")
        return cls(field=field, **kwargs)
